##################### generated by xml-casa (v2) from msuvbinflag.xml ###############
##################### ee56599cfec6ed7433808e6b2bd337c8 ##############################
from __future__ import absolute_import
import numpy
from casatools.typecheck import CasaValidator as _val_ctor
_pc = _val_ctor( )
from casatools.coercetype import coerce as _coerce
from casatools.errors import create_error_string
from .private.task_msuvbinflag import msuvbinflag as _msuvbinflag_t
from casatasks.private.task_logging import start_log as _start_log
from casatasks.private.task_logging import end_log as _end_log
from casatasks.private.task_logging import except_log as _except_log

class _msuvbinflag:
    """
    msuvbinflag ---- An algorithm to identify outliers in the UV plane through defined uniform grids by msuvbin.
    Both msuvbin and msuvbinflag tasks are experimental
    

    
    Msuvbinflag is an automatic flagging algorithm for the identification of Radio Frequency Interference (RFI)
    in the UV plane for experimental purpose. The visibilities in input CASA Measurements Set (MS) is
    first gridded into an uniformed UV plane with msuvbin task.
    Then the significant outliers are computed for entire UV grid based on the settings configurable by users for
    three method options: radian, regional mean and gradient. After computing flags for each visibility in each
    uniformed UV bin, the visibilities are transformed back to original order in the MS file with flagging information.
    The first msuvbinflag release version is for NRAO internal users to evaluate the performance for both
    computational and numerical analysis.
    
    

    --------- parameter descriptions ---------------------------------------------

    binnedvis  Name of input uniform binned visibility data (MS) set on UV plane using msuvbin task.
               default: none;
               example: binnedvis='SNR_Binnedvis.ms'
    method     There are four algorithm options for selecting the outliers as flags (radial. regionalMean, peak,gradient)
               method='radial':  This algorithm is converted the uv in cartesian coordinates into polarization coordinates along with
                                 frequency and weighting data. Then average values are calculated for each annular circle with one unit equal spacing.
                                 A binary linear least square fitting is generated followed by the comparison analysis between the original value and the
                                 fitting value. A point is treated as an outliers to be flagged if original value is greater than the sum of
                                 the fitting value and the multiple of the user defined sigma and the interpolated value.
                                 eg: originalValue[x,y] > radiafitValue[r]+sigma*interpolateValue[r].
               
               method='regionalMean': This algorithm is to compute the statistical mean, median and standard deviation for every point
                                      as a center within a square region which the size is defined by user. A point is treated as an
                                      outliers to be flagged if it is beyond the certain criteria threshold.
                                      eg: originalValue[x,y]) > regionalMean+sigma*regionalStd
               
               method='median': This algorithm is to flag according to median criteria.
               
               method='gradient': This algorithm is to compute the gradient for every point. It is usually called after the peak falgging algorithm.
                                   Then a point is treated as an outliers to be flagged if the gradient is greater than the user
                                  defined sigma. eg: gradient > sigmaGradient. After running through second time msuvbin, another comparison between
                                   user defined size of square regional median to total median, a point is treated as an outliers to be flagged if
                                   it is beyond the certain criteria threshold. eg: originalValue[x,y] > sigma * regionalStd[x,y]*median/regionalMedian.
               
               default: radial
               example: method='radial'
    sigma      User defined sigma value for comparison for radia algorithm. It is double type.
               The default value is 5.
    sizeRegion User defined local square regional size for "regionalMean" and "gradient" algorithm option.
               It is more like the size of a moving window in Numpy 2D convolution.
    radius     User defined radius value for gradient algorithm. It is double type.
               The default value is 1.
    ignorPoint User defined scale value for gradient comparison for gradient algorithm. It is bool type.
               The default value is False.
    [1;42mRETURNS[1;m       void

    --------- examples -----------------------------------------------------------

    
    
    
    msuvbinflag(binnedvis='SNR_G55_10s.ms', improveImp=True, method='radial', sigma=5)
    
    Keyword arguments:
    binnedvis -- Name of input binned visibility file which is output from msuvbin.
    After msuvbinflag, binnedvis is modified with flagging info.
    
    method -- Three method options: "radian", "regional mean","median","gradient"
    default: 'radian'
    
    sigmaRadian -- User set sigma value for radian algorithm.
    default=5.0
    
    


    """

    _info_group_ = """manipulation, imaging"""
    _info_desc_ = """An algorithm to identify outliers in the UV plane through defined uniform grids by msuvbin.
      Both msuvbin and msuvbinflag tasks are experimental
  """

    def __call__( self, binnedvis, method='radial', sigma=float(5), sizeRegion=float(20), radius=float(1), ignorPoint=True ):
        schema = {'binnedvis': {'type': 'cStr', 'coerce': _coerce.to_str}, 'method': {'type': 'cVariant', 'coerce': [_coerce.to_variant] # <allowed> IS NOT ALLOWED FOR A PARAMETER OF TYPE any
}, 'sigma': {'type': 'cFloat', 'coerce': _coerce.to_float}, 'sizeRegion': {'type': 'cFloat', 'coerce': _coerce.to_float}, 'radius': {'type': 'cFloat', 'coerce': _coerce.to_float}, 'ignorPoint': {'type': 'cBool'}}
        doc = {'binnedvis': binnedvis, 'method': method, 'sigma': sigma, 'sizeRegion': sizeRegion, 'radius': radius, 'ignorPoint': ignorPoint}
        assert _pc.validate(doc,schema), create_error_string(_pc.errors)
        _logging_state_ = _start_log( 'msuvbinflag', [ 'binnedvis=' + repr(_pc.document['binnedvis']), 'method=' + repr(_pc.document['method']), 'sigma=' + repr(_pc.document['sigma']), 'sizeRegion=' + repr(_pc.document['sizeRegion']), 'radius=' + repr(_pc.document['radius']), 'ignorPoint=' + repr(_pc.document['ignorPoint']) ] )
        task_result = None
        try:
            task_result = _msuvbinflag_t( _pc.document['binnedvis'], _pc.document['method'], _pc.document['sigma'], _pc.document['sizeRegion'], _pc.document['radius'], _pc.document['ignorPoint'] )
        except Exception as exc:
            _except_log('msuvbinflag', exc)
            raise
        finally:
            task_result = _end_log( _logging_state_, 'msuvbinflag', task_result )
        return task_result

msuvbinflag = _msuvbinflag( )

